@@ -113,6 +113,7 @@ public class AbstractParser implements Parser, Serializable {
 
   protected int fields;
 
+  protected static final int OP_NOT_LITERAL = -3;
   protected static final int OP_OVERFLOW = -2;
   protected static final int OP_TERMINATE = -1;
   protected static final int OP_RESET_FRAME = 0;
@@ -2407,8 +2408,6 @@ public class AbstractParser implements Parser, Serializable {
          * need to stop if this is not a literal.
          */
         if (compileMode && !tk.isLiteral()) {
-
-
           splitAccumulator.push(tk, new OperatorNode(operator2, expr, st, pCtx));
           return OP_OVERFLOW;
         }
@@ -2416,6 +2415,7 @@ public class AbstractParser implements Parser, Serializable {
         dStack.push(operator = operator2, tk.getReducedValue(ctx, ctx, variableFactory));
 
         while (true) {
+          ASTNode previousToken = tk;
           // look ahead again
           if ((tk = nextToken()) != null && (operator2 = tk.getOperator()) != -1
               && operator2 != END_OF_STMT && PTABLE[operator2] > PTABLE[operator]) {
@@ -2428,7 +2428,12 @@ public class AbstractParser implements Parser, Serializable {
             /**
              * This operator is of higher precedence, or the same level precedence.  push to the RHS.
              */
-            dStack.push(operator = operator2, nextToken().getReducedValue(ctx, ctx, variableFactory));
+            ASTNode nextToken = nextToken();
+            if (compileMode && !nextToken.isLiteral()) {
+              splitAccumulator.push(nextToken, new OperatorNode(operator2, expr, st, pCtx));
+              return OP_OVERFLOW;
+            }
+            dStack.push(operator = operator2, nextToken.getReducedValue(ctx, ctx, variableFactory));
 
             continue;
           }
@@ -2498,6 +2503,10 @@ public class AbstractParser implements Parser, Serializable {
               }
 
               default:
+                if (compileMode && !tk.isLiteral()) {
+                  stk.push(operator, tk);
+                  return OP_NOT_LITERAL;
+                }
                 stk.push(operator, tk.getReducedValue(ctx, ctx, variableFactory));
             }
           }
