@@ -123,7 +123,7 @@ public class ExpressionCompiler extends AbstractParser {
 
       fields |= COMPILE_IMMEDIATE;
 
-      while ((tk = nextToken()) != null) {
+      main_loop: while ((tk = nextToken()) != null) {
         /**
          * If this is a debug symbol, just add it and continue.
          */
@@ -199,7 +199,7 @@ public class ExpressionCompiler extends AbstractParser {
                     stk.push(tkLA2.getLiteralValue(), op = tkOp2.getOperator());
 
                     if (isArithmeticOperator(op)) {
-                      compileReduce(op, astBuild);
+                      if (!compileReduce(op, astBuild)) continue main_loop;
                     }
                     else {
                       reduce();
@@ -327,7 +327,7 @@ public class ExpressionCompiler extends AbstractParser {
 
   private boolean compileReduce(int opCode, ASTLinkedList astBuild) {
     switch (arithmeticFunctionReduction(opCode)) {
-      case -1:
+      case OP_TERMINATE:
         /**
          * The reduction failed because we encountered a non-literal,
          * so we must now back out and cleanup.
@@ -341,7 +341,7 @@ public class ExpressionCompiler extends AbstractParser {
             verify(pCtx, (ASTNode) splitAccumulator.pop())
         );
         return false;
-      case -2:
+      case OP_OVERFLOW:
         /**
          * Back out completely, pull everything back off the stack and add the instructions
          * to the output payload as they are.
@@ -353,6 +353,13 @@ public class ExpressionCompiler extends AbstractParser {
         astBuild.addTokenNode(new LiteralNode(stk.pop(), pCtx), operator);
         astBuild.addTokenNode(rightValue, (OperatorNode) splitAccumulator.pop());
         astBuild.addTokenNode(verify(pCtx, (ASTNode) splitAccumulator.pop()));
+        return false;
+      case OP_NOT_LITERAL:
+        ASTNode tkLA2 = (ASTNode) stk.pop();
+        Integer tkOp2 = (Integer) stk.pop();
+        astBuild.addTokenNode(new LiteralNode(getStackValueResult(), pCtx));
+        astBuild.addTokenNode(new OperatorNode(tkOp2, expr, st, pCtx), verify(pCtx, tkLA2));
+        return false;
     }
     return true;
   }
